<?php

namespace UnicaenZimbra\Processus;

use UnicaenZimbra\Entity\Account;
use UnicaenZimbra\Entity\Identity;
use UnicaenZimbra\Entity\Grant;
use UnicaenZimbra\Entity\Right;
use UnicaenZimbra\Entity\Filter\Filter;
use UnicaenZimbra\Entity\Filter\Test\Address as FilterTestAddress;
use UnicaenZimbra\Entity\Filter\Test\Header as FilterTestHeader;
use UnicaenZimbra\Entity\Filter\Action\FileInto as FilterActionFileInto;
use UnicaenZimbra\Entity\Filter\Action\Keep as FilterActionKeep;
use UnicaenZimbra\Options\ModuleOptions;
use UnicaenZimbra\Service\Folder as FolderService;
use UnicaenZimbra\Service\ZimbraGalService;

class ZimbraBoiteGeneriqueProcessus extends AbstractProcessus
{
    const ZIMBRA_TEMPORISATION = 5000000; // microsecondes

    /**
     * @var ModuleOptions
     */
    protected $options;


    /**
     * @param ModuleOptions $moduleOptions
     */
    public function setOptions(ModuleOptions $moduleOptions)
    {
        $this->options = $moduleOptions;
    }

    /**
     * @return ModuleOptions
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * Créer une boite générique
     *
     * @param string $mail Adresse mail de la boite générique
     * @param string $password Mot de passe de la boite
     * @param string $name Nom de la boite
     * @param string $description Description
     * @return self
     */
    public function create($mail, $password, $name, $description)
    {
        $accountService = $this->getZimbraAccountService();
        $cosService = $this->getZimbraCosService();

        $account = new Account;
        $account->name = $mail;
        $account->description = $description;
        $account->zimbraCOSId = $this->getOptions()->getBoiteGeneriqueCosId();
        $accountService->create($account, $password);
        $accountService->addAlias($account, $name . substr($mail, strpos($mail, '@')));
        return $this;
    }

    /**
     * Renommer une boîte générique
     *
     * @param string $mail adresse mail de la boîte générique
     * @param string $newName nouveau nom
     * @return self
     */
    public function rename($mail, $newName)
    {
        $accountService = $this->getZimbraAccountService();
        $accountService->rename($accountService->get($mail), $newName);
        return $this;
    }

    /**
     * Supprimer une boite générique
     *
     * @param string $mail Adresse mail de la boite générique
     * @return self
     */
    public function delete($mail)
    {
        $accountService = $this->getZimbraAccountService();
        $accountService->delete($accountService->get($mail));
        return $this;
    }

    /**
     * Créer une adresse fonctionnelle par rapport à la boite générique (paramètre $mail)
     *
     * @param string $mail Adresse mail de la boite générique
     * @param string $adresseFonctionnelle Adresse mail de l'adresse fonctionnelle
     * @return self
     */
    public function createAdresseFonctionnelle($mail, $adresseFonctionnelle)
    {
        $afName = $this->getMailName($adresseFonctionnelle);

        // Récupération des services
        $accountService = $this->getZimbraAccountService();
        $distribListService = $this->getZimbraDistributionListService();
        $folderService = $this->getZimbraFolderService();
        $filterService = $this->getZimbraFilterService();

        // Travail sur la boite générique
        $bg = $accountService->get($mail);
        $folderService->setAccount($bg);
        $filterService->setAccount($bg);

        // Liste de distribution
        $distribList = $distribListService->create($adresseFonctionnelle);
        usleep(self::ZIMBRA_TEMPORISATION); // temporisation nécessaire pour Zimbra

        // Ajout des membres à la liste
        $distribListService->addMembers($distribList, $mail);

        // Répertoires
        $inputFolder = $folderService->create($afName . ' | Réception');
        $sentFolder = $folderService->create($afName . ' | Envoi');

        // Filtre de réception
        $filter = new Filter;
        $filter->setName($afName . ' | Réception')
            ->setCondition($filter::CONDITION_ANYOF);
        $test1 = new FilterTestAddress;
        $test1->setHeader('to,cc')->setStringComparison('contains')->setPart('all')
            ->setValue($adresseFonctionnelle);
        $test2 = new FilterTestHeader;
        $test2->setHeader('X-Zimbra-DL')->setStringComparison('contains')
            ->setValue($adresseFonctionnelle);
        $filter->setTests([$test1, $test2]);
        $test3 = new FilterTestHeader;
        $test3->setHeader('Resent-To')->setStringComparison('contains')
            ->setValue($adresseFonctionnelle);
        $test4 = new FilterTestHeader;
        $test4->setHeader('Received')->setStringComparison('contains')
            ->setValue("for <$adresseFonctionnelle>");
        $filter->setTests([$test1, $test2, $test3, $test4]);

        $action = new FilterActionFileInto;
        $action->setFolderPath($inputFolder->absFolderPath);
        $filter->addAction($action);
        $filter->addAction(new FilterActionKeep);
        $filterService->createInput($filter);

        return $this;
    }

    /**
     * Renommer une adresse fonctionnelle
     *
     * @param string $mail Adresse mail de la boite générique
     * @param string $adresseFonctionnelle adresse mail de l'adresse fonctionnelle
     * @param string $newName nouveau nom
     * @return self
     */
    public function renameAdresseFonctionnelle($mail, $adresseFonctionnelle, $newName)
    {
        $afName = $this->getMailName($adresseFonctionnelle);
        $afNewName = $this->getMailName($newName);

        // récupération des services
        $accountService = $this->getZimbraAccountService();
        $distribListService = $this->getZimbraDistributionListService();
        $folderService = $this->getZimbraFolderService();
        $filterService = $this->getZimbraFilterService();

        // Travail sur la boite générique
        $bg = $accountService->get($mail);
        $folderService->setAccount($bg);
        $filterService->setAccount($bg);

        // Liste de distribution
        // la suppression de la liste de distribution permet de supprimer les droits (ACE) sur celle-ci
        $distribListService->delete($distribListService->get($adresseFonctionnelle));
        $distribList = $distribListService->create($newName);
        usleep(self::ZIMBRA_TEMPORISATION); // temporisation nécessaire pour Zimbra

        // Ajout des membres à la liste
        $distribListService->addMembers($distribList, $mail);

        // Répertoires
        $inputFolder = $folderService->rename($folderService->get($afName . ' | Réception'), $afNewName . ' | Réception');
        $sentFolder = $folderService->rename($folderService->get($afName . ' | Envoi'), $afNewName . ' | Envoi');

        // Filtre entrant
        $filter = new Filter;
//	    $filter = $filterService->getInput()[$afName.' | Réception'];
        $filter->setName($afNewName . ' | Réception')
            ->setCondition($filter::CONDITION_ANYOF);
        $test1 = new FilterTestAddress();
        $test1->setHeader('to,cc')->setStringComparison('contains')->setPart('all')
            ->setValue($newName);
        $test2 = new FilterTestHeader();
        $test2->setHeader('X-Zimbra-DL')->setStringComparison('contains')
            ->setValue($newName);
        $test3 = new FilterTestHeader;
        $test3->setHeader('Resent-To')->setStringComparison('contains')
            ->setValue($newName);
        $test4 = new FilterTestHeader;
        $test4->setHeader('Received')->setStringComparison('contains')
            ->setValue("for <$newName>");
        $filter->setTests([$test1, $test2, $test3, $test4]);

        $action = new FilterActionFileInto;
        $action->setFolderPath($afNewName . ' | Réception');
        $filter->addAction($action);
        $filter->addAction(new FilterActionKeep);

        $filterService->deleteInput($afName . ' | Réception');
        $filterService->createInput($filter);

        return $this;
    }

    /**
     * Supprimer une adresse fonctionnelle par rapport à la boite générique (paramètre $mail)
     *
     * @param string $mail Adresse mail de la boite générique
     * @param string $adresseFonctionnelle Adresse mail de l'adresse fonctionnelle
     * @return self
     */
    public function deleteAdresseFonctionnelle($mail, $adresseFonctionnelle)
    {
        $afName = $this->getMailName($adresseFonctionnelle);

        // récupération des services
        $accountService = $this->getZimbraAccountService();
        $distribListService = $this->getZimbraDistributionListService();
        $folderService = $this->getZimbraFolderService();
        $filterService = $this->getZimbraFilterService();

        // Travail sur la boite générique
        $bg = $accountService->get($mail);
        $folderService->setAccount($bg);
        $filterService->setAccount($bg);

        // Liste de distribution
        $distribListService->delete($distribListService->get($adresseFonctionnelle));

        // Répertoires
        $folderService->delete($folderService->get($afName . ' | Réception'));
        $folderService->delete($folderService->get($afName . ' | Envoi'));

        // Filtres
        $filterService->deleteInput($afName . ' | Réception');

        return $this;
    }

    /**
     * Retourner les droits sur une adresse fonctionnelle
     * Permet notamment de récupérer les utilisateurs qui ont un accès à l'adresse fonctionnelle
     *
     * @param type $adresseFonctionnelle Adresse mail de l'adresse fonctionnelle
     * @return Right[]
     */
    public function getGrantsOfAdresseFonctionnelle($adresseFonctionnelle)
    {
        $distribListService = $this->getZimbraDistributionListService();
        $rightService = $this->getZimbraRightService();

        $ace = new Right;
        $ace->setTarget($distribListService->get($adresseFonctionnelle));
        return $rightService->getGrants($ace);
    }

    /**
     * Donner accès à une adresse fonctionnelle pour un utilisateur
     *
     * @param string $adresseFonctionnelle Adresse mail de l'adresse fonctionnelle
     * @param string $userMail Adresse mail de l'utilisateur
     * @return self
     */
    public function grantUserToAdresseFonctionnelle($adresseFonctionnelle, $userMail)
    {
        $afName = $this->getMailName($adresseFonctionnelle);

        $distribListService = $this->getZimbraDistributionListService();
        $accountService = $this->getZimbraAccountService();
        $folderService = $this->getZimbraFolderService();
        $filterService = $this->getZimbraFilterService();
        $rightService = $this->getZimbraRightService();
        $identityService = $this->getZimbraIdentityService();

        // Travail sur la boite générique
        $dlAccount = $distribListService->get($adresseFonctionnelle);
        foreach($dlAccount->members as $member) {
            if(preg_match('/bg-.*/', $member)) {
                $bgAccount = $member;
                break;
            }
        }
        $bgAccount = $bgAccount ?: $dlAccount->members[0];

        $userAccount = $accountService->get($userMail);
        $folderService->setAccount($bgAccount);

        // Partages
        $inputFolder = $folderService->get($afName . ' | Réception');
        $sentFolder = $folderService->get($afName . ' | Envoi');

        $grant = new Grant;
        $grant->setGrantee($userAccount);
        $grant->perm = Grant::PERM_MANAGE;

        $folderService->addGrant($inputFolder, $grant, FolderService::GRANT_ACTION_NONE)
            ->createMountPoint($inputFolder, $userAccount, $inputFolder->absFolderPath)
            ->addGrant($sentFolder, $grant, FolderService::GRANT_ACTION_NONE)
            ->createMountPoint($sentFolder, $userAccount, $sentFolder->absFolderPath);

        // Travail sur la boite utilisateur
        $filterService->setAccount($userAccount);
        $identityService->setAccount($userAccount);
        $folderService->setAccount($userAccount);

        // Filtre d'envoi
        $filter = new Filter;
        $filter->setName($afName . ' | Envoi')
            ->setCondition($filter::CONDITION_ANYOF);
        $test1 = new FilterTestAddress;
        $test1->setHeader('from')->setStringComparison('contains')->setPart('all')
            ->setValue($adresseFonctionnelle);
        $test2 = new FilterTestHeader();
        $test2->setHeader('Reply-To')->setStringComparison('contains')
            ->setValue($adresseFonctionnelle);
        $filter->setTests([$test1, $test2]);

        $action = new FilterActionFileInto;
        $action->setFolderPath($sentFolder->absFolderPath);
        $filter->addAction($action);
        $filterService->createOutput($filter);

        // ACE
        $ace = new Right;
        $ace->setTarget($dlAccount)
            ->setGrantee($userAccount)
            ->setRightByRightName('sendAsDistList');
        $rightService->grantRight($ace);

        // Avatar
        $identity = new Identity;
        $identity->setName($afName);
        $identity->zimbraPrefFromAddress = $adresseFonctionnelle;
        $identity->zimbraPrefFromAddressType = 'sendAs';
        $identity->zimbraPrefFromDisplay = $afName;
        $identity->zimbraPrefWhenInFolderIds = $folderService->get($afName . ' | Réception')->getId();
        $identity->zimbraPrefWhenInFoldersEnabled = true;
        $identity->zimbraPrefReplyToAddress = $adresseFonctionnelle;
        $identity->zimbraPrefReplyToEnabled = true;
        $identity->zimbraPrefWhenSentToAddresses = $adresseFonctionnelle;
        $identity->zimbraPrefWhenSentToEnabled = true;
        $identityService->create($identity);

        return $this;
    }

    /**
     * Retirer l'accès à une adresse fonctionnelle pour un utilisateur
     *
     * @param string $adresseFonctionnelle Adresse mail de l'adresse fonctionnelle
     * @param string $userMail Adresse mail de l'utilisateur
     * @return self
     */
    public function revokeUserFromAdresseFonctionnelle($adresseFonctionnelle, $userMail)
    {
        $afName = $this->getMailName($adresseFonctionnelle);

        $distribListService = $this->getZimbraDistributionListService();
        $accountService = $this->getZimbraAccountService();
        $folderService = $this->getZimbraFolderService();
        $filterService = $this->getZimbraFilterService();
        $rightService = $this->getZimbraRightService();
        $identityService = $this->getZimbraIdentityService();

        // Travail sur la boite générique
        $dlAccount = $distribListService->get($adresseFonctionnelle);
        foreach($dlAccount->members as $member) {
            if(preg_match('/bg-.*/', $member)) {
                $bgAccount = $member;
                break;
            }
        }
        $userAccount = $accountService->get($userMail);
        $folderService->setAccount($bgAccount);

        // Suppression des partages
        $inputFolder = $folderService->get($afName . ' | Réception');
        $sentFolder = $folderService->get($afName . ' | Envoi');

        $folderService->revokeGrant($inputFolder, $userAccount)
            ->revokeGrant($sentFolder, $userAccount);

        // Travail sur la boite utilisateur
        $filterService->setAccount($userAccount);
        $identityService->setAccount($userAccount);

        // Filtres
        $filterService->deleteOutput($afName . ' | Envoi');

        // ACE
        $ace = new Right;
        $ace->setTarget($dlAccount)
            ->setGrantee($userAccount)
            ->setRightByRightName('sendAsDistList');
        $rightService->revokeRight($ace);

        // Avatar
        $identityService->delete($identityService->get($afName));
        return $this;
    }

    /**
     * Renommer une adresse fonctionnelle pour un utilisateur
     *
     * @param string $adresseFonctionnelle adresse mail de l'adresse fonctionnelle
     * @param type $userMail adresse mail de l'utilisateur
     * @param type $newName nouveau nom
     * @return self
     */
    public function renameAdresseFonctionnelleToUser($adresseFonctionnelle, $userMail, $newName)
    {
        $afName = $this->getMailName($adresseFonctionnelle);
        $afNewName = $this->getMailName($newName);

        $distribListService = $this->getZimbraDistributionListService();
        $accountService = $this->getZimbraAccountService();
        $folderService = $this->getZimbraFolderService();
        $filterService = $this->getZimbraFilterService();
        $rightService = $this->getZimbraRightService();
        $identityService = $this->getZimbraIdentityService();

        // Travail sur la boite utilisateur
        $userAccount = $accountService->get($userMail);
        $folderService->setAccount($userAccount);
        $filterService->setAccount($userAccount);
        $identityService->setAccount($userAccount);

        // Répertoires
        $inputFolder = $folderService->rename($inputFolder = $folderService->get($afName . ' | Réception'), $afNewName . ' | Réception');
        $sentFolder = $folderService->rename($sentFolder = $folderService->get($afName . ' | Envoi'), $afNewName . ' | Envoi');

        // Filtre sortant
        $filter = new Filter;
//	    $filter = $filterService->getOutput()[$afName.' | Envoi'];
        $filter->setName($afNewName . ' | Envoi');
        $test = new FilterTestAddress();
        $test->setHeader('from')->setStringComparison('contains')->setPart('all')
            ->setValue($newName);
        $filter->setTests([$test]);

        $action = new FilterActionFileInto;
        $action->setFolderPath($afNewName . ' | Envoi');
        $filter->addAction($action);

        $filterService->deleteOutput($afName . ' | Envoi');
        $filterService->createOutput($filter);

        // ACE
        $newAce = new Right();
        $newAce->setTarget($distribListService->get($newName))
            ->setGrantee($userAccount)
            ->setRightByRightName('sendAsDistList');
        $rightService->grantRight($newAce);

        // Avatar
        $identityService->delete($identityService->get($afName)); // suppression ancienne identité
        $identity = new Identity();
        $identity->setName($afNewName);
        $identity->zimbraPrefFromAddress = $newName;
        $identity->zimbraPrefFromAddressType = 'sendAs';
        $identity->zimbraPrefFromDisplay = $afNewName;
        $identity->zimbraPrefWhenInFolderIds = $folderService->get($afNewName . ' | Réception')->getId();
        $identity->zimbraPrefWhenInFoldersEnabled = true;
        $identity->zimbraPrefReplyToAddress = $newName;
        $identity->zimbraPrefReplyToEnabled = true;
        $identity->zimbraPrefWhenSentToAddresses = $newName;
        $identity->zimbraPrefWhenSentToEnabled = true;
        $identityService->create($identity);

        return $this;
    }

    /**
     * Retourne la partie locale d'une adresse mail (tout ce qu'il y a avant le dernier @)
     *
     * @param string $mail
     * @return string
     */
    private function getMailName($mail)
    {
        return substr($mail, 0, strrpos($mail, '@'));
    }
}